#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       webDriverAgent.py
Create time:    2019/5/24 14:28
Description:

"""

import os
import socket
import subprocess
import re
import os.path
from sys import stderr
import requests
import time
import logging
import shutil
import platform

logger = logging.getLogger()
isWindows = "Windows" in platform.platform()


def exec_cmd(cmd):
    process = subprocess.Popen(
        cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True
    )
    return process.pid


def exec_iproxy(cmd):
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    for line in iter(process.stdout.readline, b""):
        line = line.rstrip().decode("utf8")
        logger.debug(line)
        if "waiting for connection" in line:
            # time.sleep(3)
            break
    return process.pid

def exec_cmd_until(cmd, expected_output):
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
    for line in iter(process.stdout.readline, b""):
        line = line.rstrip().decode("utf8")
        logger.debug(line)
        m = re.search(expected_output, line)
        if m:
            return process.pid, m.group(0)
    return process.pid, ""


def do_shell(command, print_msg=True):
    """
    执行 shell 语句
    :param command:
    :param print_msg:
    :return:
    """
    p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    if print_msg:
        logger.debug(command)
    lines = []
    for line in iter(p.stdout.readline, b""):
        line = line.rstrip().decode("utf8")
        if print_msg:
            print(line)
        lines.append(line)
        if "ServerURLHere->http://" in line:
            # time.sleep(5)
            break
    return p.pid


IOS_DEVICES = {
    "i386": "iPhone Simulator",
    "x86_64": "iPhone Simulator",
    "iPhone3,1": "iPhone 4",
    "iPhone3,3": "iPhone 4 (Verizon)",
    "iPhone4,1": "iPhone 4S",
    "iPhone5,1": "iPhone 5 (GSM)",
    "iPhone5,2": "iPhone 5 (GSM+CDMA)",
    "iPhone5,3": "iPhone 5C (GSM)",
    "iPhone5,4": "iPhone 5C (Global)",
    "iPhone6,1": "iPhone 5S (GSM)",
    "iPhone6,2": "iPhone 5S (Global)",
    "iPhone7,1": "iPhone 6 Plus",
    "iPhone7,2": "iPhone 6",
    "iPhone8,1": "iPhone 6s",
    "iPhone8,2": "iPhone 6s Plus",
    "iPhone8,3": "iPhone SE (GSM+CDMA)",
    "iPhone8,4": "iPhone SE (GSM)",
    "iPhone9,1": "iPhone 7",
    "iPhone9,2": "iPhone 7 Plus",
    "iPhone9,3": "iPhone 7",
    "iPhone9,4": "iPhone 7 Plus",
    "iPhone10,1": "iPhone 8",
    "iPhone10,2": "iPhone 8 Plus",
    "iPhone10,3": "iPhone X",
    "iPhone11,2": "iPhone XS",
    "iPhone11,4": "iPhone XS Max",
    "iPhone11,6": "iPhone XS Max",
    "iPhone11,8": "iPhone XR",
    "iPad1,1": "iPad 1",
    "iPad2,1": "iPad 2 (WiFi)",
    "iPad2,2": "iPad 2 (GSM)",
    "iPad2,3": "iPad 2 (CDMA)",
    "iPad2,4": "iPad 2 (WiFi)",
    "iPad2,5": "iPad Mini (WiFi)",
    "iPad2,6": "iPad Mini (GSM)",
    "iPad2,7": "iPad Mini (GSM+CDMA)",
    "iPad3,1": "iPad 3 (WiFi)",
    "iPad3,2": "iPad 3 (GSM+CDMA)",
    "iPad3,3": "iPad 3 (GSM)",
    "iPad3,4": "iPad 4 (WiFi)",
    "iPad3,5": "iPad 4 (GSM)",
    "iPad3,6": "iPad 4 (GSM+CDMA)",
    "iPad4,1": "iPad Air (WiFi)",
    "iPad4,2": "iPad Air (GSM+CDMA)",
    "iPad4,4": "iPad Mini Retina (WiFi)",
    "iPad4,5": "iPad Mini Retina (GSM+CDMA)",
    "iPad4,6": "iPad mini Retina (China)",
    "iPad4,7": "iPad mini 3 (WiFi)",
    "iPad4,8": "iPad mini 3 (GSM+CDMA)",
    "iPad4,9": "iPad Mini 3 (China)",
    "iPad5,3": "iPad Air 2 (WiFi)",
    "iPad5,4": "iPad Air 2 (Cellular)",
    "iPad6,3": "iPad Pro (9.7 inch, Wi-Fi)",
    "iPad6,4": "iPad Pro (9.7 inch, Wi-Fi+LTE)",
    "iPad6,7": "iPad Pro (12.9 inch, Wi-Fi)",
    "iPad6,8": "iPad Pro (12.9 inch, Wi-Fi+LTE)",
}


class DeviceToolException(Exception):
    def __init__(self, value):
        self._value = value

    def __str__(self):
        return repr(self._value)


class DeviceTool(object):
    def __init__(self, udid=None):
        self.udid = udid
        if self.udid is None:
            self.udid = self.get_default_udid()

    def exec_cmd(self, cmd, output=False):
        if output:
            process = subprocess.Popen(
                cmd,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=True,
            )
            return process.communicate()[0].strip()
        else:
            subprocess.call(cmd, shell=True)

    def list_devices(self):
        cmd = "idevice_id -l"
        devices = self.exec_cmd(cmd, True)
        device_list = []
        if len(devices) != 0:
            device_list = devices.split("\n")
        return device_list

    def get_default_udid(self):
        devices = self.list_devices()
        if len(devices) == 0:
            raise DeviceToolException("没有设备连接...")

        return devices[0]

    def list_user_app(self):
        cmd = "ideviceinstaller -u %s -l -o list_user" % self.udid
        return self.exec_cmd(cmd, True)

    def get_property(self, key):
        cmd = "ideviceinfo -u %s -k %s" % (self.udid, key)
        return self.exec_cmd(cmd, True)

    @property
    def name(self):
        return self.get_property("DeviceName")

    @property
    def type(self):
        t = IOS_DEVICES.get(self.get_property("ProductType"))
        if t != None:
            return t
        else:
            return "unkown"

    @property
    def os_version(self):
        return self.get_property("ProductVersion")

    @property
    def region(self):
        return self.get_property("RegionInfo")

    @property
    def timezone(self):
        return self.get_property("TimeZone")

    @property
    def desc(self):
        return {
            "manu": "Apple",
            "name": self.name,
            "model": self.type,
            "version": self.os_version,
        }

    def find_app(self, bundle_id="com.tencent.xin"):
        ids = []
        ids_wetest = {}
        installed_apps = self.list_user_app()
        for id in installed_apps.decode("utf-8").split("\n"):
            if re.search("-", id):
                str = id[0 : id.find("-")].strip()
                ids.append(str)
            if re.search(",", id):
                str_wetest = id[0 : id.find(",")].strip()
                ids_wetest[str_wetest] = id.split(", ")[1][1:-1]

        if bundle_id in ids or bundle_id in ids_wetest.keys():
            logger.info(
                "检测到已安装 %s 版本的微信" % ids_wetest[bundle_id]
            ) if bundle_id in ids_wetest.keys() else logger.info("微信已安装...")
            return True
        else:
            logger.error("检测到设备未安装微信, 请前往 APP store 安装最新版微信...")
            return False

    def screenshot(self, filename):
        cmd = "idevicescreenshot -u %s '%s'" % (self.udid, filename)
        return self.exec_cmd(cmd, True)

    def get_crashes(self, dir, app_name="WeChat"):
        """
        获取指定app的crash文件列表
        比如：
            WeChat-2018-06-13-105022.ips (计入crash)
            WeChat.wakeups_resource-2018-06-13-005223.ips (使用资源太多太频繁才被系统kill, 不计入crash)
        """
        cmd = "idevicecrashreport -k -u %s %s" % (self.udid, dir)
        report = self.exec_cmd(cmd, True)
        result = []
        for line in report.split("\n"):
            if "%s-" % app_name in line:
                index = line.index("W")
                value = line[index:].strip()
                result.append(value)
        return result

    def remove_crashes(self, dir):
        cmd = "idevicecrashreport -u %s %s" % (self.udid, dir)
        return self.exec_cmd(cmd, True)


class WebDriverRunner(object):
    def __init__(self, device_id, driver_path):
        self.device_id = device_id
        self.driver_path = driver_path
        self.port = 8100  # self.pick_unuse_port()
        self.iproxy_pid = None
        self.listen_port(port=self.port, device_id=self.device_id)

    def pick_unuse_port(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(("localhost", 0))
        addr, port = s.getsockname()
        s.close()
        self.port = port
        return port

    def listen_port(self, port=None, device_id=None):
        if not port:
            port = self.pick_unuse_port()

        if not device_id:
            device_id = self.device_id
        logger.warning(
            "iproxy command has been updated, you need to run 'brew upgrade usbmuxd' in your"
            " computer"
        )
        cmd = "iproxy {port}:8100 -u {device_id}"
        cmd = cmd.format(port=port, device_id=device_id)
        self.iproxy_pid = exec_cmd(cmd)
        logger.info(f"[{self.iproxy_pid}]{cmd}")
        time.sleep(1)

    def remove_iproxy(self):
        cmd = "kill -9 `ps -ef|grep iproxy|grep -v grep|grep -v kill|awk '{print $2}'`"
        logger.debug(cmd)
        exec_cmd(cmd)
        time.sleep(2)

    def start_driver(self):
        logger.info("driver_path: %s" % self.driver_path)

        base_cmd = "/Applications/Xcode.app/Contents/Developer/usr/bin/xcodebuild"

        driver_data_home = os.path.join(self.driver_path, self.device_id)
        if os.path.exists(driver_data_home):
            shutil.rmtree(driver_data_home)
        log_dir = self.get_log_dir()
        cmd = (
            base_cmd
            + " -project {driver_path}/WebDriverAgent.xcodeproj -scheme WebDriverAgentRunner"
            " -derivedDataPath {driver_path}/{device_id} -destination 'id={device_id}' test &"
        )
        cmd = cmd.format(driver_path=self.driver_path, device_id=self.device_id, log_dir=log_dir)
        do_shell(cmd)

    def kill_driver(self):
        cmd = "pkill -f id=%s" % self.device_id
        exec_cmd(cmd)

    def ping_driver(self, timeout):
        url = "http://localhost:%s/status" % self.port

        try:
            res = requests.get(url, timeout=timeout)
            if res.status_code == requests.codes.ok:
                logger.info("WebDriver在线")
                return True
        except:
            logger.info("获取不到Driver状态...")
            return False

    def wait_for_driver_ready(self, timeout=100):
        s = time.time()
        while time.time() - s < timeout:
            time.sleep(10)
            if self.ping_driver(self.port, 5):
                break
        else:
            message = "%d秒后，仍获取不到Driver状态，请检查..." % timeout
            logger.error(message)
            raise RuntimeError(message)

    def get_log_dir(self):
        log_dir = os.path.join(os.path.dirname(__file__), "../log")
        if not os.path.exists(log_dir):
            os.makedirs(log_dir)
        return log_dir


class TIDevice(object):
    def __init__(self, device_id, wda_bundle=None):
        """
        :wda_bundle: wda的bundle id
        """
        self.device_id = device_id
        if not device_id:
            self.device_id = self.get_default_device()
        else:
            self.device_id = device_id
        if not self.device_id:
            raise RuntimeError("未检测到任何设备")
        if not wda_bundle:
            self.wda_bundle = self.get_wda_bundle_id()
        else:
            self.wda_bundle = wda_bundle
        if not self.wda_bundle:
            raise RuntimeError("未检测到设备上的WebDriverAgentRunner, 请确认已经安装")
        self.port = 8100  # self.pick_unuse_port()
        self.iproxy_pid = None
        self.run_xctest()
        self.listen_port()

    def check_port(self, port):
        """
        检查端口是不是一个wda proxy
        """
        try:
            session = requests.Session()
            session.trust_env = False  # localhost不需要代理
            session.get("http://127.0.0.1:{port}/status")
        except requests.HTTPError as he:
            return False
        return True

    def pick_unuse_port(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(("localhost", 0))
        addr, port = s.getsockname()
        s.close()
        self.port = port
        return port

    def exec_cmd(self, cmd, output=False):
        if output:
            process = subprocess.Popen(
                cmd,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=True,
            )
            return process.communicate()[0].strip()
        else:
            subprocess.call(cmd, shell=True)

    def get_default_device(self):
        """
        获取默认设备id
        `tidevice list` return like `12308101-001A08903AA0023Y abc's iPhone12P`
        """
        output = self.exec_cmd("tidevice list", True)
        lines = output.decode("utf8").split("\n")
        if len(lines) == 0 or lines[0].strip() == "":
            return None
        return lines[0].split(" ")[0]

    def get_wda_bundle_id(self):
        """
        如果没有传入wda_bundle, 可以通过applist识别可能的bundle id
        `tidevice applist` return like `com.netease.cloudmusic 网易云音乐 6.4.6`
        """
        cmd = "tidevice"
        if self.device_id:
            cmd += " -u %s" % self.device_id
        cmd += " applist"
        output = self.exec_cmd(cmd, True).decode("utf8")
        lines = output.split("\n")
        for line in lines:
            line = line.strip()
            if not line:
                continue
            re_match = re.match("(\S+)\s(.*)", line)
            if not re_match:
                continue
            bundle, name = re_match.groups()
            if re.match("com\..*\.xctrunner", bundle):  #  com.*.xctrunner
                return bundle
            if name.find("WebDriverAgentRunner") >= 0:
                return bundle
        return None

    def run_xctest(self, device_id=None):
        if not device_id:
            device_id = self.device_id
        # tidevice -u 00008101-001A08903AA0001E xctest --bundle_id "com.*.xctrunner"
        # check whether xctest running
        if not isWindows:
            code = os.system(f"""ps -ef|grep 'tidevice -u "{device_id}" xctest'|grep -v grep""")
            if code == 0:  # exists
                return
        cmd = 'tidevice -u "{device_id}" xctest --bundle_id "{wda_bundle}"'
        cmd = cmd.format(device_id=device_id, wda_bundle=self.wda_bundle)
        pid, output = exec_cmd_until(cmd, r"(Launch failed|No app matches|WebDriverAgent start successfully)")
        if output == "Launch failed":
            raise Exception("Launch WebDriverAgent for xctest fail")
        elif output == "No app matches":
            raise Exception("Can't find app[%s], please ensure that you have install it" % self.wda_bundle)
        elif output == "WebDriverAgent start successfully":
            return True
        time.sleep(1)

    def listen_port(self, port=None, device_id=None):
        if not port:
            port = self.pick_unuse_port()

        if not device_id:
            device_id = self.device_id
        logger.warning("use tidevice")
        cmd = 'tidevice -u "{device_id}" relay {port} 8100'
        cmd = cmd.format(port=port, device_id=device_id)
        self.iproxy_pid = exec_cmd(cmd)
        logger.info(f"[{self.iproxy_pid}]{cmd}")
        time.sleep(1)

    def remove_iproxy(self):
        """
        remove tidevice wdaproxy
        """
        cmd = (
            'kill -9 `ps -ef|grep "tidevice -u %s relay"|grep -v grep|grep -v kill|awk \'{print'
            " $2}'`"
            % self.device_id
        )
        logger.debug(cmd)
        exec_cmd(cmd)
        time.sleep(2)

    def start_driver(self):
        self.run_xctest()


if __name__ == "__main__":
    runner = WebDriverRunner(
        device_id="00008020-000445EE3684002E",
        driver_path="/Users/sherlock/github/WebDriverAgent",
    )
    device = DeviceTool(udid="00008020-000445EE3684002E")
    device.find_app("com.tencent.xin")



# encoding=utf-8
import json
import time
import traceback
import logging
import threading
import websocket
from .wspools import WsPools
from .error import *


logger = logging.getLogger()

MAX_WAIT_TIMEOUT = 50 #原先33
OPEN_TIMEOUT     = 4
CLOSE_TIMEOUT    = 4


class TabWebSocket(object):
    "与chrome内核的一个tab标签的Websocket连接"
    _ID_COUNTER = 0

    def __init__(self, url, desc=None, tab_id=None, title=None):
        self._identity = TabWebSocket._ID_COUNTER
        TabWebSocket._ID_COUNTER += 1
        self._url = url
        self._desc = desc
        self._tab_id = tab_id
        self._title = title
        self._open_lock = threading.Condition()
        self._close_lock = threading.Condition()
        self._msg_lock = threading.Condition()
        self._thread = None
        self._init()
        self.ensure_opened()

    def _init(self):
        self._width = -1
        self._height = -1

        self._req_id_counter = 0  # message id
        self._ws_event_queue = dict()
        self._is_running = False  # 标记连接是否建立
        self._status = "close"
        self._sync_wait_msg_id = None
        self._sync_wait_msg = None


    # ==============================================================================================
    # Property
    # ==============================================================================================
    @property
    def identity(self): return self._identity

    @property
    def url(self): return self._url

    @property
    def desc(self):
        return self._desc

    @property
    def tab_id(self):
        return self._tab_id

    @property
    def title(self):
        return self._title

    def is_opened(self):
        return self._status == 'open'

    # ==============================================================================================
    # WebSocket Event Private Handler
    # ==============================================================================================
    def _on_open(self, ws=None):
        """
        :type ws: websocket.WebSocketApp
        """
        logger.debug('%s, %s, %s, opened', ws, self.url, self._thread.ident)
        self._status = "open"
        self.set_running(True)
        self._open_lock.acquire()
        self._open_lock.notify()
        self._open_lock.release()

    def _on_message(self, ws=None, message=None):
        """
        :type ws: websocket.WebSocketApp
        """
        logger.debug("%s, %s, %s: %.1024s", ws, self.url, self._thread.ident, message)
        msg = json.loads(message)
        if msg is not None and "id" in msg:  # response
            req_id = msg["id"]
            if req_id == self._sync_wait_msg_id:
                self._sync_wait_msg_id = None
                self._sync_wait_msg = msg
                self._msg_lock.acquire()
                self._msg_lock.notify()
                self._msg_lock.release()
            else:
                logger.warning('abandon msg: %s', req_id)
        else: # event from X5
            if "method" in msg and "params" in msg:
                self._push_event(msg["method"], msg["params"])

    def _on_error(self, ws=None, error=None):
        logger.error("%s, %s, %s, error %s", ws, self.url, self._thread.ident, error)

    def _on_close(self, ws=None):
        self.set_running(False)
        self._status = "close"
        logger.debug('%s, %s, closed in on_close', self.url, self._thread.ident)

        self._close_lock.acquire()
        self._close_lock.notify()
        self._close_lock.release()

    # ==============================================================================================
    # Open & Close
    # ==============================================================================================
    def _ws_run_forever(self):
        "在其他线程中执行"
        url = self.url
        logger.debug("%s, %s, ready to run", url, self._thread.ident)

        try:
            self._client.run_forever()
            self._init()
        except:
            self.set_running(False) # TODO 重构
            traceback.print_exc()
            logger.exception('websocket run error')
            return

        logger.debug("%s, %s, run forerver shutdown", url, self._thread.ident)

    def ensure_opened(self, timeout=OPEN_TIMEOUT):
        if self._status != "close":
            return
        WsPools.add(self)   # 成对出现
        self._client = websocket.WebSocketApp(self.url, on_open=self._on_open, on_message=self._on_message,
                                              on_error=self._on_error, on_close=self._on_close)
        self._open_lock.acquire()
        self._thread = threading.Thread(target=self._ws_run_forever, args=())
        self._status = "pending"
        self._thread.daemon = True
        self._thread.start()
        self._open_lock.wait(timeout) # 等待on_open唤醒
        self._open_lock.release()

        if self._is_running:
            logger.debug("%s, open in open API", self.url)
        else:
            logger.debug('url: %s', self.url)
            raise OpenWebSocketException('can not open websocket')

    def close(self, timeout=CLOSE_TIMEOUT):
        "关闭当前连接，幂等，允许多次调用"
        if not WsPools.has(self.identity): return
        WsPools.remove(self)    # 成对出现
        self._client.close()
        self._close_lock.acquire()
        self._close_lock.wait(timeout)   # 等待on_close唤醒
        self._close_lock.release()

        if self._is_running: # 有的机器close和on_close就不是成对出现的
            logger.warning('timeout, can not close websocket')
            #   raise CloseWebSocketException('timeout, can not close weboscket')
        else:
            logger.debug("%s, closed in close API", self.url)

    def __del__(self):
        self.close()

    def set_running(self, is_running):
        self._is_running = is_running

    # ==============================================================================================
    # Raw Event Protocol
    # ==============================================================================================
    def _push_event(self, method, params):
        if method in self._ws_event_queue:
            self._ws_event_queue[method].append(params)
        else:
            self._ws_event_queue[method] = [params, ]

    def pop_event(self, method):
        "弹出队列里先来的消息"
        if method in self._ws_event_queue:
            params = self._ws_event_queue[method][0]
            if len(self._ws_event_queue[method]) == 1:
                del self._ws_event_queue[method]
            else:
                del self._ws_event_queue[method][0]
            return params
        raise RuntimeError('no event: %s in queue' % method)

    def peek_event(self, method):
        "获取队列里先来的消息, 但不弹出"
        if self.has_event(method):
            return self._ws_event_queue[method][0]
        raise RuntimeError('no event: %s in queue' % method)

    def has_event(self, method):
        "判断事件消息是否存在"
        return method in self._ws_event_queue and len(self._ws_event_queue[method]) > 0

    def get_event(self, method):
        "获取事件队列"
        if method in self._ws_event_queue:
            return self._ws_event_queue[method]
        else:
            return None

    # TODO 使用锁来重构
    def wait_for_event(self, method, timeout=MAX_WAIT_TIMEOUT):
        "等待事件返回"
        start_time = time.time()
        while not self.has_event(method) and self._is_running:
            if time.time() - start_time > timeout:
                logger.warn("timeout " + method)
                break
            time.sleep(0.1)
        if self.has_event(method):
            return self.peek_event(method)
        return None

    # ==============================================================================================
    # Raw Communicate Protocol
    # ==============================================================================================
    def async_request(self, domain, command, params=None):
        "不阻塞异步执行, 不需要返回数据"
        req = { "method": "%s.%s" % (domain, command), "params": params }
        req_id = self._send_request(req)
        return req_id

    def sync_request(self, domain, command, params=None, timeout=MAX_WAIT_TIMEOUT):
        "阻塞直到获取返回数据, 或者超时报错"
        req = { "method": "%s.%s" % (domain, command), "params": params }
        self._sync_wait_msg_id = self._send_request(req)
        # logger.info(self._sync_wait_msg_id)
        return self._receive_request(timeout)

    def _send_request(self, params):
        if "id" in params:
            req_id = params["id"]
        else:
            params["id"] = req_id = self._get_req_id()
        serialize = json.dumps(params)
        # logger.debug('send: %s', serialize)
        self.ensure_opened()
        self._client.send(serialize)
        return req_id

    def _receive_request(self, timeout=MAX_WAIT_TIMEOUT):
        self._msg_lock.acquire()
        self._msg_lock.wait(timeout)
        self._msg_lock.release()

        if self._sync_wait_msg_id is None: # 获取到了数据
            return self._sync_wait_msg
        else:
            record_id = self._sync_wait_msg_id
            self._sync_wait_msg_id = None
            self._sync_wait_msg = None
            raise ResponseTimeout("receive from remote timeout, id: %s" % record_id)

    def _get_req_id(self):
        self._req_id_counter += 1
        return self._req_id_counter

    # ==============================================================================================
    # Runtime Protocol
    # ==============================================================================================
    def run_script(self, script, timeout=MAX_WAIT_TIMEOUT):
        # 小程序monkey的部分逻辑用到了includeCommandLineAPI
        # 协议版本: 1.1, 很可能不支持awaitPromise
        data = self.sync_request("Runtime", "evaluate", {"expression": script, "includeCommandLineAPI": True}, timeout)

        # TODO 如果webSocket提前中断, data可能是None
        if data is None: raise WebSocketLoseError("can not get script return, maybe webSocket crash happened")
        return data

    def run_script_with_output(self, script, timeout=MAX_WAIT_TIMEOUT):
        data = self.run_script(script, timeout)

        # 观察一下, 因为有前面的守护判断, 这里的data应该总是合法
        result = data["result"]["result"]
        result_type = result["type"]

        if result_type in ["boolean", "string", "number"]:
            return result["value"]
        else:
            logger.debug(result_type)
        return None

    def await_promise(self, promise_object_id, timeout = MAX_WAIT_TIMEOUT):
        "不用使用, 1.1不支持"
        data = self.sync_request('Runtime', 'awaitPromise', {"promiseObjectId": promise_object_id}, timeout)
        return data

    # ==============================================================================================
    # DOM Protocol
    # ==============================================================================================
    def get_document(self):
        data = self.sync_request('DOM', 'getDocument')
        return data['result']['root']

    def query_selector(self, node_id, selector):
        data = self.sync_request('DOM', 'querySelector', { 'nodeId': node_id, 'selector': selector })
        return data['result']

    def request_node(self, object_id):
        data = self.sync_request('DOM', 'requestNode', { 'objectId': object_id })
        return data

    def get_attributes(self, node_id):
        data = self.sync_request('DOM', 'getAttributes', { 'nodeId': node_id })
        return data

    # ==============================================================================================
    # Page Protocol
    # ==============================================================================================
    def get(self, url):
        self.sync_request("Page", "navigate", {"url": url})

    def reload(self, ignore_cache=True):
        self._ws_event_queue = dict()
        self.sync_request("Page", "reload", {"ignoreCache": ignore_cache})

    def page_enable(self):
        self.sync_request("Page", "enable", {})

    def network_enable(self):
        self.sync_request("Network", "enable", {})

    def network_clear_browser_cache(self):
        self.sync_request("Network", "clearBrowserCache", {})

    def network_set_cache_disabled(self):
        self.sync_request("Network", "setCacheDisabled", {"cacheDisabled": True})

    def console_enable(self, is_enable=True):
        if is_enable:
            method = "enable"
        else:
            method = "disable"
        self.sync_request("Console", method)


# self.errors = []
# def error(self, msg):
#     self.errors.append(msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Created by xiazeng on 2017/6/13
from __future__ import print_function
import time
import threading
import re
import logging
from at import At


logger = logging.getLogger(__name__)

class TopCollector(threading.Thread):
    def __init__(self, at, interval=0.5):
        """
        :type at: At
        :param interval:
        """
        super(TopCollector, self).__init__()
        self.at = at
        self._interval = interval
        self._should_stop = False
        self._cpu_process_names = None
        self._mem_process_names = None
        self._flow_pkg_names = None
        self._pre_shell_cmd = None
        self.top_datas = []

    def stop(self):
        self._should_stop = True
        self.join()

    def run(self) -> None:
        while not self._should_stop:
            output = self.at.adb.run_shell("top -H -b -n 1")
            self.add_output(output)

    def add_output(self, output):
        lines = output.split("\n")
        is_thread_line = False
        summary = {
            "mem": {
                "total": 0,
                "used": 0,
                "free": 0,
                "buffers": 0
            },
            "swap":{
                "total": 0,
                "used": 0,
                "free": 0,
                "cached": 0
            },
            "cpu": {

            }
        }
        threads = []
        item = {
            "time": time.time(),
            "summary": summary,
            "threads": threads
        }
        mem_r = re.compile(r"Mem:\s+(\d+)k total,\s+(\d+)k used,\s+(\d+)k free,\s+(\d+)k buffers")
        swap_r = re.compile(r"Swap:\s+(\d+)k total,\s+(\d+)k used,\s+(\d+)k free,\s+(\d+)k cached")
        cpu_r = re.compile(r"(\d+)%cpu\s+(\d+)%user\s+(\d+)%nice\s+(\d+)%sys\s+(\d+)%idle.*")
        for line in lines:
            line = line.strip()
            if not line:
                continue
            elif "PID USER" in line:
                is_thread_line = True
            elif is_thread_line:
                ls = re.split(r"\s+", line)
                #  1059 u0_a19       10 -10 2.6G 374M 158M S 91.0  19.0 147:15.43 ncent.wxpayface com.tencent.wxpayface
                if len(ls) < 13:
                    logger.error("unknown line: %s", line)
                else:
                    threads.append({
                        "VIRT": ls[4],
                        "RES": ls[5],
                        "SHR": ls[6],
                        "cpu": ls[8],
                        "mem": ls[9],
                        "time+": ls[10],
                        "thread": ls[11],
                        "process": ls[12]
                    })
            else:
                if mem_r.search(line):
                    m = mem_r.search(line)
                    summary['mem']['total'] = m.group(1)
                    summary['mem']['used'] = m.group(2)
                    summary['mem']['free'] = m.group(3)
                    summary['mem']['buffers'] = m.group(4)
                elif swap_r.search(line):
                    m = swap_r.search(line)
                    summary['swap']['total'] = m.group(1)
                    summary['swap']['used'] = m.group(2)
                    summary['swap']['free'] = m.group(3)
                    summary['swap']['cached'] = m.group(4)
                elif cpu_r.search(line):
                    m = cpu_r.search(line)
                    summary['cpu']['total'] = m.group(1)
                    summary['cpu']['user'] = m.group(2)
                    summary['cpu']['nice'] = m.group(3)
                    summary['cpu']['sys'] = m.group(4)
                else:
                    logger.error("unknown:%s", line)
        self.top_datas.append(item)


class PerfCollector(threading.Thread):
    def __init__(self, at, interval=2, nativemem = False):
        super(PerfCollector, self).__init__()
        self.at = at
        self._interval = interval
        self._should_stop = False
        self._cpu_process_names = None
        self._mem_process_names = None
        self._vmsize_process_names = None
        self._fps_process_names = None
        self._flow_pkg_names = None
        self._pre_shell_cmd = None
        self.data = {}
        self.nativemem = nativemem

    def collect_cpu(self, *process_names):
        self._cpu_process_names = process_names

    def collect_mem(self, *process_names):
        self._mem_process_names = process_names

    def collect_vmsize(self,*process_names):
        self._vmsize_process_names = process_names

    def collect_fps(self, *process_names):
        self._fps_process_names = process_names

    def collect_flow(self, *pkg_names):
        self._flow_pkg_names = pkg_names

    def stop(self):
        self._should_stop = True
        for i in range(self._interval):
            if not self.isAlive():
                break
            time.sleep(1)

    def run(self):
        while not self._should_stop:
            if self._pre_shell_cmd:
                if isinstance(self._pre_shell_cmd, str):
                    self.at.adb.run_shell(self._pre_shell_cmd)
                else:
                    for cmd in self._pre_shell_cmd:
                        self.at.adb.run_shell(cmd)

            if self._cpu_process_names:
                ret = self.at.adb.get_cpu_rate(*self._cpu_process_names)
                ts = int(time.time())
                for k, v in ret.items():
                    if "cpu" not in self.data:
                        self.data["cpu"] = {}
                    if k not in self.data["cpu"]:
                        self.data["cpu"][k] = []
                    self.data["cpu"][k].append((ts, v))

            if self._vmsize_process_names:
                ret_vmsize = self.at.adb.get_vmsize(*self._vmsize_process_names)
                ts = int(time.time())
                for k, v in ret_vmsize.items():
                    if "vmsize" not in self.data:
                        self.data["vmsize"] = {}
                    if k not in self.data["vmsize"]:
                        self.data["vmsize"][k] = []

                    self.data["vmsize"][k].append((ts, v))
            if self._mem_process_names:
                if self.nativemem:
                    ret = self.at.adb.get_mem_used_native(*self._mem_process_names)
                    ts = int(time.time())
                    for k, v in ret.items():
                        if "mem" not in self.data:
                            self.data["mem"] = {}
                        if k not in self.data["mem"]:
                            self.data["mem"][k] = []
                        self.data["mem"][k].append((ts, v[0], v[1], v[2], v[3], v[4], v[5]))
                else:
                    ret = self.at.adb.get_mem_used(*self._mem_process_names)
                    ts = int(time.time())
                    for k, v in ret.items():
                        if "mem" not in self.data:
                            self.data["mem"] = {}
                        if k not in self.data["mem"]:
                            self.data["mem"][k] = []
                        self.data["mem"][k].append((ts, v))
            if self._fps_process_names:
                ret = self.at.adb.get_fps_rate(*self._fps_process_names)
                ts = int(time.time())
                for k, v in ret.items():
                    if "fps" not in self.data:
                        self.data["fps"] = {}
                    if k not in self.data["fps"]:
                        self.data["fps"][k] = []
                    self.data["fps"][k].append((ts, v))
            if self._flow_pkg_names:
                for pkg_name in self._flow_pkg_names:
                    m_rx, m_tx, wifi_rx, wifi_tx = self.at.adb.get_pkg_traffic_stats(pkg_name)
                    ts = int(time.time())
                    if "flow" not in self.data:
                        self.data["flow"] = {}
                    if pkg_name not in self.data["flow"]:
                        self.data["flow"][pkg_name] = []
                    self.data["flow"][pkg_name].append((ts, m_rx, m_tx, wifi_rx, wifi_tx))
            time.sleep(self._interval)

    def register_pre_shell_cmd(self, cmd):
        if not isinstance(cmd, str):
            raise RuntimeError("Invalid args")

        if not self._pre_shell_cmd:
            self._pre_shell_cmd = cmd
        elif isinstance(self._pre_shell_cmd, str):
            self._pre_shell_cmd = [self._pre_shell_cmd, cmd]
        elif isinstance(self._pre_shell_cmd, list):
            self._pre_shell_cmd.append(cmd)


def test_top(serial):
    at = At(serial)
    c = TopCollector(at)
    c.setDaemon(True)
    c.start()
    for i in range(1):
        time.sleep(1)
    import json
    json.dump(c.top_datas, open("top_data.json", "w"), indent=4)
    c.stop()


def run(serial):
    at = At(serial)
    c = PerfCollector(at)
    process = ["com.tencent.mm", "com.tencent.mm:appbrand0"]
    c.collect_cpu(*process)
    c.collect_mem(*process)
    c.setDaemon(True)
    c.start()
    for i in range(1200):
        # print c.data
        for key in c.data:
            for process_name in c.data[key]:
                print(key, process_name)
                values = c.data[key][process_name]
                print(', '.join([str(v[1]) for v in values]))
        time.sleep(5)
    c.stop()


if __name__ == '__main__':
    import logging
    logging.basicConfig(level=logging.DEBUG)
    test_top("TXAP11951001223ND002112")


#!/usr/bin/env python
# encoding:utf-8
import logging
from attestbase import AtTestBase

logger = logging.getLogger()


class JavaDriverTest(AtTestBase):
    def test_dump_ui(self):
        ui_views = self.at.java_driver.dump_ui()
        self.assertTrue(len(ui_views)>1)
        for ui_view in ui_views:
            logger.info(ui_view)

    def test_screen_shot(self):
        self.at.device.screen_shot("front.jpg", display_id=1)
        self.at.device.screen_shot("back.jpg", display_id=0)


#!/usr/bin/env python
# encoding:utf-8
import unittest
import logging
import os.path
import at.vision.template_match
logging.basicConfig(level=logging.DEBUG)

logger = logging.getLogger()

root = os.path.join(os.path.dirname(__file__), "images")


class AtTestBase(unittest.TestCase):
    def test_match(self):
        source_path = os.path.join(root, "image.jpg")
        in_path = os.path.join(root, "2.jpg")
        r = at.vision.template_match.search(in_path, source_path)
        print(r)


#!/usr/bin/env python3
# Created by xiazeng on 2019-05-22
import json
import logging
import threading
import platform
from .basenative import BaseNative
from ..lib.wx_wda import *
from .wording import WORDING, Language

tidevice = None
OTHER_PACKAGE = "com.others.app"
WECHAT_PACKAGE = "com.tencent.xin"
WECHAT_ACTIVITY = "MainFrameViewController"
MINIPROGRAM_ACTIVITY = "WAWebViewController"
OTHER_ACTIVITY = "OtherActivity"

logger = logging.getLogger("minium")
if "Windows" in platform.platform():
    isWindows = True
else:
    isWindows = False


class WXIOSNative(BaseNative):
    _require_conf_ = [
        # ("wda_project_path", "wda_bundle")
    ]
    useTIDevice = False

    def __init__(self, json_conf: dict):
        if json_conf is None:
            json_conf = {}
        super(WXIOSNative, self).__init__(json_conf)
        # 目标设备, 目标app
        self.udid = json_conf.get("device_info", {}).get("udid")
        self.bundle_id = json_conf.get("device_info", {}).get(
            "bundle_id", WECHAT_PACKAGE
        )
        # 使用xcode + wda project需要以下信息
        self.wda_project_path = json_conf.get("wda_project_path")
        self.wda_runner = None
        # 使用tidevice
        self.wda_bundle = json_conf.get("wda_bundle", None)
        # 目标app实例
        self.app = None
        # 获取性能的实例
        self.perf_flag = False
        self.perf = None
        self.perf_data = []
        self.last_cpu = 0
        self.last_fps = 0
        self.last_mem = 0
        # 健康弹窗处理
        self.health_modal_handle_thread = None
        self.stop_check = True
        self.check_done = True
        # screen_shot
        self.outputs_screen = os.path.join(
            json_conf.get("outputs") or os.path.dirname(os.path.realpath(__file__)),
            "image",
        )
        self._empty_base_screen_dir(self.outputs_screen)
        # 分享小程序相关
        self._forward_wording = None
        self._new_chat_wording = None

    ###############################
    #                    interface                        #
    ###############################
    def start_wechat(self):
        """
        启动微信
        :return:
        """
        if self.wda_project_path and not isWindows:
            self.wda_runner = WebDriverRunner(self.udid, self.wda_project_path)
        else:
            self.wda_runner = TIDevice(self.udid, self.wda_bundle)
            self.useTIDevice = True
        for i in range(3):
            try:
                logger.info("第 %d 次启动微信, 共3次机会" % (i + 1))
                self.app = WdaUI(
                    server_url="http://localhost:%s" % self.wda_runner.port,
                    bundle_id=self.bundle_id,
                )
                self.app.session.set_alert_callback(self._alert_callback) if callable(
                    self._alert_callback
                ) else logger.error("Alert callback would not callable")
                logger.info("微信启动成功")
                # 重启多次可能会进入安全模式
                if self.app.session(
                    class_name="NavigationBar", text="安全模式"
                ).wait_exists(timeout=5.0):
                    while self.app.session(
                        class_name="Button", text="下一步"
                    ).click_if_exists(timeout=3.0):
                        time.sleep(1)
                    self.app.session(class_name="Button", text="进入微信").click_if_exists(
                        timeout=3.0
                    )
                lan = True
                for text in WORDING.COMMON.LOGIN_WORDS.zh:
                    if not self.app.session(text=text).exists:
                        lan = False
                        break
                if lan is False:
                    lan = True
                    for text in WORDING.COMMON.LOGIN_WORDS.en:
                        if not self.app.session(text=text).exists:
                            lan = False
                            break
                    if lan is True:
                        WORDING.setLanguage(Language.en)
                else:
                    WORDING.setLanguage(Language.zh)
                return
            except Exception as e:
                if i == 2:
                    # e.args += "setup error: 第 %d 次启动微信失败" % (i + 1)
                    raise
                logger.error("setup error: 第 %d 次启动微信失败: %s" % ((i + 1), str(e)))
                if "Connection refused" in str(e):
                    logger.warning(
                        "Connection refused, 端口[%s]不可用，重新选择 iproxy 端口"
                        % self.wda_runner.port
                    )
                    self.wda_runner.remove_iproxy()
                    port = self.wda_runner.pick_unuse_port()
                    self.wda_runner.listen_port(port)
                    continue
                logger.info("正在重启 WebDriverAgent ...")
                self.wda_runner.start_driver()

    def connect_weapp(self, path):
        """
        有push的方式, 理应可以废掉
        """
        raise NotImplementedError("ios不再支持长按识别二维码的方式，请使用推送方式调用")
        filename = "remote_debug.jpg"
        r = requests.post(
            "https://stream.weixin.qq.com/weapp/UploadFile",
            files={filename: (filename, open(path, "rb"))},
        )
        if r.status_code != 200:
            logger.error(r.text)
            r.raise_for_status()

        url = "https://stream.weixin.qq.com/weapp/GetQRCodePage?file_name={0}".format(
            filename
        )
        self.app.session(class_name="Button", text="通讯录").click(timeout=10.0)
        self.app.session(class_name="SearchField", text="搜索").set_text("文件传输助手")
        self.app.session(xpath="//Table[1]/Cell[2]/Button[1]").click(timeout=10)
        self.app.session(class_name="TextView").set_text(url + "\n")
        self.app.session(class_name="Other", partial_text=url)[-1].click(timeout=10.0)
        self.app.session(class_name="WebView").subelems(
            class_name="Other", text="小程序web view测试"
        ).subelems(class_name="Image").tap_hold(3.0)
        self.app.session(partial_text="识别图中二维码").click(timeout=10.0)

    def screen_shot(self, filename: str, return_format: str = "raw") -> object:
        """
        截图
        :param filename: 文件存放的路径
        :param return_format: 除了将截图保存在本地之外, 需要返回的图片内容格式: raw(default) or pillow
        :return: raw data or PIL.Image
        """
        try:
            self.app.client.screenshot(png_filename=filename)
            return filename
        except Exception as e:

            logger.warning("screen shot failed, %s" % e)

    def pick_media_file(
        self,
        cap_type="camera",
        media_type="photo",
        original=False,
        duration=5.0,
        names=None,
    ):
        """
        获取媒体文件
        :param cap_type: camera: 拍摄 | album: 从相册获取
        :param names: 传入一个 list 选择多张照片或者视频(照片和视频不能同时选择, 且图片最多 9 张, 视频一次只能选择一个)
        :param media_type: photo 或者 video
        :param duration: 拍摄时长
        :param original: 是否选择原图(仅图片)
        :return:
        """
        if cap_type == "album" and names is None:
            raise Exception("从相册选择照片必须提供照片名称, 可以通过 wda inspector 查看照片名称")
        if cap_type == "camera":
            self._capture_photo(media_type=media_type, duration=duration)
        elif cap_type == "album":
            if media_type == "photo":
                if isinstance(names, str):
                    names = [names]
                self._select_photos_from_album(names=names, original=original)
            elif media_type == "video":
                if isinstance(names, list):
                    names = names[0]
                self._select_video_from_album(name=names)

    def input_text(self, text):
        """
        input 组件填写文字(使用此函数之前必须确保输入框处于输入状态)
        :param text: 内容
        :return:
        """
        self.app.session(class_name="TextField").set_text(text)

    def input_clear(self):
        """
        input 组件清除文字(使用此函数之前必须确保输入框处于输入状态)
        :return:
        """
        self.app.session(class_name="TextField").clear_text()

    def textarea_text(self, text: str, index=0):
        """
        给 textarea 输入文字需要在小程序层提供该 textarea 的 index 信息
        :param text: 内容
        :param index: 多个 textarea 同时存在一个页面从上往下排序, 计数从 1 开始
        :return:
        """
        self.app.session(class_name="TextView")[index].set_text(text)

    def textarea_clear(self, index=0):
        """
        给 textarea 清除文字需要在小程序层提供该 textarea 的 index 信息
        :param index: 多个 textarea 同时存在一个页面从上往下排序, 计数从 1 开始
        :return:
        """
        self.app.session(class_name="TextView")[index].clear_text()

    def allow_authorize(self, answer=True, title=None):
        """
        处理授权确认弹框
        :param answer: True or False
        :return:
        """
        logger.debug("handle Button[%s], title[%s]" % ("允许" if answer else "拒绝", title))
        if title:
            return self.app.session(
                xpath='//*[contains(@label, "{title}")]/../../..//Button[@name="{text}"]'.format(
                    title=title, text="允许" if answer else "拒绝"
                )
            ).click_if_exists(timeout=10.0)
        else:
            return self.app.session(
                xpath='//Button[@label="授权说明"]/../../..//Button[@name="{text}"]'.format(
                    text="允许" if answer else "拒绝"
                )
            ).click_if_exists(timeout=10.0)

    def allow_login(self, answer=True):
        """
        处理微信登陆确认弹框
        :return:
        """
        if answer:
            self.app.session(class_name="Button", text="允许").click_if_exists(
                timeout=10.0
            )
        else:
            self.app.session(class_name="Button", text="拒绝").click_if_exists(
                timeout=10.0
            )

    def allow_get_user_info(self, answer=True):
        """
        处理获取用户信息确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "获取你的昵称")

    def allow_get_location(self, answer=True):
        """
        处理获取位置信息确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "获取你的位置信息")

    def allow_get_we_run_data(self, answer=True):
        """
        处理获取微信运动数据确认弹框
        :param answer: True: 允许 or False: 拒绝
        :return:

        备注：
        - 未启用微信运动时，启用微信运动后就立即授权使用了，无须再次授权允许
        - 开启微信运动后，ios可能会有健康弹窗，出现时间不定
        处理策略：
        1. 未开启微信运动，且授权操作为“否”(answer==False)：不开启微信运动，返回false
        2. 未开启微信运动，且授权操作为“是”(answer==True)：开启微信运动，监测健康弹窗
        3. 已经开启过微信运动：监测健康弹窗，允许微信获取健康数据，再在小程序授权弹窗中再处理对app的授权与否
        4. 如果曾经不允许微信获取健康数据，调用getWeRunData时，会弹modal窗"暂不能获取完整健康数据"，即使从系统拿不到,但数据还是能从后台拿, 如果有授权弹窗，返回{answer}，否则默认True
        5. 无授权弹窗出现，默认同意，返回True
        """
        # 开启微信运动后，直接就默认了授权使用微信运动了
        if self.app.session(class_name="NavigationBar", text="开启微信运动").wait_exists(
            timeout=3.0
        ):
            if not answer:
                self.app.session(text="关闭").click_if_exists(timeout=10)
                return False
            self.app.session(class_name="Button", text="启用该功能").click_if_exists(
                timeout=10
            )
        # 监测 "暂不能获取完整健康数据" 弹窗, 如果存在，则系统层面已经不允许获取健康数据，返回False
        if self.modal_exists("暂不能获取完整健康数据"):
            logger.error("暂不能获取完整健康数据")
            self.handle_modal("确定", title="暂不能获取完整健康数据")
            # 此时授权弹窗可能已经弹了出来, 即使从系统拿不到,但数据还是能从后台拿
            ret = self.allow_authorize(answer, "获取你的微信运动步数")
            return answer if ret else True
        # 开启之后ios可能会有健康弹窗，出现时间不定
        self.health_page_exists = False
        if not self.health_modal_handle_thread:

            def handle_health_modal():
                # print("handle_health_modal 1 %f" % time.time())
                while self.app.session.alert.exists:
                    self.app.session.alert.accept()
                    time.sleep(1)
                # print("handle_health_modal 2 %f" % time.time())
                if not self.app.search_text("想要访问并更新以下类别中的健康数据"):
                    print("handle_health_modal 3 %f" % time.time())
                    return False
                # print("health_page_exists wait %f" % time.time())
                self.health_page_exists = True
                switch = self.app.session(class_name="Switch", index=-1)
                if not self._wait(lambda: switch.exists, 10):
                    logger.error("Switch not exists")
                    return False
                if str(switch.value) == "0":
                    switch.click(timeout=5)
                accept = self.app.session(class_name="Button", text="允许", index=-1)
                if not self._wait(lambda: accept.exists and accept.enabled, 10):
                    logger.error(
                        "accept button is %s and %s"
                        % (
                            "exists" if accept.exists else "not exists",
                            "enabled" if accept.enabled else "not enabled",
                        )
                    )
                    return False
                if not accept.click_if_exists(timeout=5):
                    logger.error("accept error: button not exists")
                    return False
                time.sleep(2)
                return True

            def check_health():
                if self._wait(
                    lambda: handle_health_modal() or self.stop_check, 3 * 60, 5
                ):  # 没有健康授权之前，点不了
                    self.health_page_exists = False
                    self.allow_authorize(answer, "获取你的微信运动步数")
                # print("finish wait %f" % time.time())
                self.check_done = True
                self.health_modal_handle_thread = None

            self.stop_check = False
            self.check_done = False
            self.health_modal_handle_thread = threading.Thread(target=check_health)
            self.health_modal_handle_thread.setDaemon(True)
            self.health_modal_handle_thread.start()
        if not self._wait(lambda: self.health_page_exists, timeout=10, interval=2):
            # print("health_page_exists wait fail %f" % time.time())
            ret = self.allow_authorize(answer, "获取你的微信运动步数")
            return answer if ret else True
        else:
            # print("start wait %f" % time.time())
            # 出现了健康授权页，就必须得等授权页处理好——系统同意授权，但真正授权由微信授权弹窗确定
            ret = self._wait(lambda: not self.health_page_exists, 3 * 60, 10)
            return answer if ret else True

    def allow_record(self, answer=True):
        """
        处理录音确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "使用你的麦克风")

    def allow_write_photos_album(self, answer=True):
        """
        处理保存相册确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "保存图片或视频到你的相册")

    def allow_camera(self, answer=True):
        """
        处理使用摄像头确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "使用你的摄像头")

    def allow_get_user_phone(self, answer=True):
        """
        处理获取用户手机号码确认弹框
        :param answer: True or False
        :return:
        """
        self.allow_authorize(answer, "你的手机号码")

    def allow_send_subscribe_message(self, answer=True):
        """
        允许发送订阅消息
        """
        if answer:
            btn_text = "允许|确定"
        else:
            btn_text = "取消"
        self.app.session(
            xpath=(
                '//StaticText[contains(@label, "以下消息")]/../../..//Button[contains("{btn_text}", @label)]'.format(
                    btn_text=btn_text
                )
            )
        ).click_if_exists(timeout=5.0)

    def modal_exists(self, title):
        """
        指定title的modal是否存在
        """
        if not self.app.session(
            xpath='//Button[.//StaticText[@name="{title}"]]'.format(title=title)
        ).exists:
            # title 和 content同时传入的时候，才能get到这个title的信息, 所以此处只有确定会有title时才做处理
            logger.info(f"没有出现预期弹窗: title[{title}]")
            return False
        return True

    def handle_modal(
        self, btn_text="确定", title: str = None, index=-1, force_title=False
    ):
        """
        处理模态弹窗
        :param title: 传入弹窗的 title 可以校验当前弹窗是否为预期弹窗
        :param btn_text: 根据传入的 文字 进行点击
        :param index: 当页面存在完全相同的两个控件时，通过指定 index 来选取
        :return:
        """
        if title and force_title:
            if not self.modal_exists(title):
                return False
        logger.info(f"可能出现弹框：{title}, 自动选择{btn_text}")
        return self.app.session(
            xpath='//Button//StaticText[@name="{btn_text}"]'.format(btn_text=btn_text),
            index=index,
        ).click_if_exists(timeout=5.0)

    def handle_action_sheet(self, item):
        """
        处理上拉菜单
        :param item: 要选择的 item
        :return:
        """
        self.app.session(class_name="ScrollView").subelems(
            class_name="Button", text=item
        ).click(timeout=10.0)

    def forward_miniprogram(
        self, name: str or list, text: str = None, create_new_chat: bool = True
    ):
        """
        通过右上角更多菜单转发小程序
        ps: 好友太多会有性能问题
        :type text: 分享携带的内容
        :param names: 要分享的人
        :param create_new_chat: 是否创建群聊
        :return:
        """
        self.app.session(class_name="Button", text=WORDING.COMMON.MORE.value).click(timeout=10.0)
        time.sleep(1)
        if self._forward_wording is None:  # 不同客户端版本有不同wording
            for word in [ WORDING.IOS.FORWARD_WORD1, WORDING.IOS.FORWARD_WORD2, WORDING.IOS.FORWARD_WORD3 ]:
                if self.app.session(class_name="Button", partial_text=word.value).exists:
                    self._forward_wording = word
                    break
        if self._forward_wording and self.app.session(class_name="Button", partial_text=self._forward_wording.value).exists:
            self.app.session(class_name="Button", partial_text=self._forward_wording.value).click(timeout=10.0)
        else:
            self.app.session(partial_text="关于").click(timeout=10.0)
            self.app.session(partial_text="推荐给朋友").click(timeout=10.0)

        return self.forward_miniprogram_inside(name, text, create_new_chat)

    def forward_miniprogram_inside(
        self, name: str or list, text: str = None, create_new_chat: bool = True
    ):
        """
        小程序内触发转发小程序
        ps: 好友太多会有性能问题
        :param names: 要分享的人
        :param create_new_chat: 是否创建群聊
        :return:
        """
        if self.app.session(
            xpath='//Button[.//StaticText[@name="分享提示"]]'
        ).exists:  # 开发版会弹出分享提示
            self.handle_modal("确定")

        if isinstance(name, str):
            name = [name]
        if len(name) > 1:  # 多于一个人是需要新建群
            create_new_chat = True
        if not create_new_chat and self.app.session(text="暂无最近聊天").exists:
            create_new_chat = True
        if create_new_chat:
            if self._new_chat_wording is None:
                for word in [ WORDING.IOS.CREATE_CHAT1, WORDING.IOS.CREATE_CHAT2 ]:
                    if self.app.session(class_name="StaticText", text=word.value).exists:
                        self._new_chat_wording = word
                        break
                if not self._new_chat_wording:
                    raise RuntimeError("无法创建新聊天")
            self.app.session(class_name="StaticText", text=self._new_chat_wording.value).click(timeout=10.0)
        for _name in name:
            self.app.session(
                class_name="TextField" if create_new_chat else "SearchField", text=WORDING.COMMON.SEARCH.value
            ).set_text(_name)
            # count = 10
            time.sleep(1.5)  # 一定需要等待搜索出该用户
            self.app.session(
                class_name="StaticText", partial_text=_name, index=-1
            ).click(timeout=10.0)
            # count -= 1
        if create_new_chat:
            self.app.session(class_name="Button", partial_text=WORDING.COMMON.DONE.value).click(timeout=10.0)
        self.app.session(class_name="Button", text=WORDING.COMMON.SEND.value).click(timeout=10.0)

    def send_custom_message(self, message: str = None):
        """
        处理小程序 im 发送自定义消息
        :param message: 消息内容
        :return:
        """
        self.app.session(class_name="TextView").set_text(message + "\n")

    def phone_call(self):
        """
        处理小程序拨打电话
        :return:
        """
        self.app.session(partial_text="呼叫").click(timeout=10.0)
        self.app.session.alert.accept()

    def map_select_location(self, name: str = None):
        """
        原生地图组件选择位置
        :param name: 位置名称
        :return:
        """
        if not name:
            btn = self.app.session(class_name="Button", text="确定")
            if btn.exists and btn.enabled:
                return btn.click(timeout=10.0)
            else:
                return self.map_back_to_mp()
        self.app.session(class_name="SearchField", text="搜索地点").set_text(name)
        timeout = time.time() + 10
        while (
            self.app.session(text=name, class_name="StaticText").click_if_exists(
                timeout=5.0
            )
            and timeout > time.time()
        ):
            btn = self.app.session(class_name="Button", text="确定")
            if btn.exists and btn.enabled:
                return self.app.session(class_name="Button", text="确定").click(
                    timeout=10.0
                )
        # 没有命中就选第一个
        timeout = time.time() + 10
        while (
            self.app.session(xpath="//Table/Cell[1]").click_if_exists(timeout=5.0)
            and timeout > time.time()
        ):
            btn = self.app.session(class_name="Button", text="确定")
            if btn.exists and btn.enabled:
                return self.app.session(class_name="Button", text="确定").click(
                    timeout=10.0
                )

    def map_back_to_mp(self):
        """
        原生地图组件查看定位,返回小程序
        :return:
        """
        return self.app.session(class_name="Button", text="取消").click_if_exists(
            timeout=10.0
        )

    def deactivate(self, duration):
        """
        使微信进入后台一段时间, 再切回前台
        :param duration: float
        :return: NULL
        """
        self.app.deactivate(duration=duration)

    def click_coordinate(self, x, y):
        """
        点击坐标(x,y)
        :param x:
        :param y:
        :return:
        """
        self.app.session.click(x, y)

    def get_pay_value(self):
        """
        获取支付金额, IOS隐私设置不允许获取
        """
        raise NotImplementedError("iOS private value")

    def input_pay_password(self):
        """
        输入支付密码, IOS隐私设置不允许自动化输入
        """
        raise NotImplementedError()

    def close_payment_dialog(self):
        """
        关闭支付弹窗
        """
        self.app.session(text="closeModalBtn").click_if_exists(timeout=5.0)

    def hide_keyboard(self):
        """
        点击完成键，隐藏键盘
        :return:
        """
        self.app.session(class_name="Button", text="Done").click_if_exists(timeout=5.0)

    def text_exists(self, text="", iscontain=False, wait_seconds=5):
        """
        检测是否存在text
        """
        if iscontain:
            return self.app.session(partial_text=text).exists
        else:
            return self.app.session(text=text).exists

    def text_click(self, text="", iscontain=False):
        """
        点击内容为text的控件
        """
        if iscontain:
            return self.app.session(partial_text=text).click_if_exists(timeout=10.0)
        else:
            return self.app.session(text=text).click_if_exists(timeout=10.0)

    def is_app_in_foreground(self, appid):
        # exists window contains appid
        return self.app.session(class_name="Window", partial_text=appid).exists

    # back_to_miniprogram needed

    def _get_current_activity(self) -> str:
        """
        :return: PACKAGE/ACTIVITY
        """
        activity = []
        app_status = self.app.session.app_state(self.bundle_id)
        if app_status.value == AppState.RUNNING_IN_FOREGROUND.value:
            activity.append(self.bundle_id)
        else:
            activity.append(OTHER_PACKAGE)
        # 检查胶囊 和 webview，同时存在则在小程序中
        capsule = self.app.session(
            xpath='//Other/Button[@label="更多"]/following-sibling::XCUIElementTypeButton[@label="关闭"]'
        )
        webview = self.app.session(class_name="WebView")
        # 检查tabbar
        tabbar_items = ["微信", "通讯录", "发现", "我"]
        main_frame_tabbar = self.app.session(
            xpath='//TabBar/XCUIElementTypeButton[@label="%s"]/' % tabbar_items[0]
            + "/".join(
                'following-sibling::XCUIElementTypeButton[@label="%s"]' % item
                for item in tabbar_items[1:]
            )
        )
        if capsule.exists and webview.exists:
            activity.append(MINIPROGRAM_ACTIVITY)
        elif main_frame_tabbar.exists:
            activity.append(WECHAT_ACTIVITY)
        else:
            activity.append(OTHER_ACTIVITY)
        return "/".join(activity)

    def _is_in_wechat(self, activity: str):
        return activity.startswith(self.bundle_id)

    def _is_in_wechat_main(self, activity: str):
        return activity.endswith(WECHAT_ACTIVITY)

    def _is_in_miniprogram(self, activity: str):
        return activity.endswith(MINIPROGRAM_ACTIVITY)

    def _get_any_modal(self, confirm=False):
        """
        confirm == True: 确认/允许
        confirm == False: 拒绝/取消
        1. 授权弹窗 —— 拒绝/允许
        2. 分享弹窗 —— 发送
        3. 模态弹窗 —— 取消/确定
        4. ACTION SHEET暂不支持
        """
        auth = self.app.session(
            xpath='//Button[@label="授权说明"]/../Button[@name="{text}"]'.format(text="允许" if confirm else "拒绝")
        )
        share = self.app.session(
            xpath='//Image/StaticText[@name="发送给："]/following-sibling::XCUIElementTypeOther/Button[@label="发送"]'
        )
        modal = self.app.session(
            xpath='//Button[@label=""]/StaticText[contains("确定|取消", @label)]'
        )
        action_sheet = self.app.session(xpath='//ScrollView/Button[@label="取消"]')
        if auth.exists:
            return auth
        if modal.exists:
            return modal
        if action_sheet.exists:
            return action_sheet
        if share.exists:
            return share

    def _press_back(self):
        return self.app.press_back()

    @property
    def orientation(self):
        """
        获取屏幕方向
        :return:
        """
        return self.app.session.orientation

    @orientation.setter
    def orientation(self, value):
        """
        设置屏幕方向
        :param value: (string) LANDSCAPE | PORTRAIT | UIA_DEVICE_ORIENTATION_LANDSCAPERIGHT |
                    UIA_DEVICE_ORIENTATION_PORTRAIT_UPSIDEDOWN
        :return:
        """
        self.app.session.orientationset(value)

    def release(self):
        """
        remove port forward process
        :return:
        """
        self.release_auto_authorize()
        if self.perf_flag:
            self.stop_get_perf()
        self.wda_runner.remove_iproxy()

    ###############################
    #                      private                         #
    ###############################

    def _capture_photo(self, media_type, duration=10.0):
        """
        capture a photo by camera
        :param media_type: photo or video
        :param duration: capture duration
        :return:
        """
        if media_type == "photo":
            self.app.session(text="拍照").click(timeout=10.0)
            self.app.session(text="轻触拍照，按住摄像").click(timeout=10.0)
        elif media_type == "video":
            self.app.session(text="拍摄").click(timeout=10.0)
            self.app.session(text="轻触拍照，按住摄像").tap_hold(duration=duration)
        time.sleep(2.0)
        while self.app.session(text="确定").exists:
            try:
                self.app.session(text="确定").click(timeout=10.0)
            except Exception as e:
                logger.warning(str(e))

    def _select_photos_from_album(self, names: list, original=False):
        """
        select photos from album
        :param names: photo name list
        :param original: use original photo or not
        :return:
        """
        self.app.session(text="从手机相册选择").click(timeout=10.0)
        for name in names:
            rect = self.app.session(partial_text=name).bounds
            self.app.session.click(rect.x + rect.width - 10, rect.y + 10)
        if original:
            self.app.session(text="原图").click(timeout=10.0)
        self.app.session(partial_text="完成").click(timeout=10.0)

    def _select_video_from_album(self, name: str):
        """
        select video from album
        :param name: video file name
        :return:
        """
        self.app.session(text="从手机相册选择").click(timeout=10.0)
        rect = self.app.session(partial_text=name).bounds
        self.app.session.click(rect.x + rect.width - 10, rect.y + 10)
        self.app.session(partial_text="完成").click(timeout=10.0)

    def stop_wechat(self):
        """
        :return:
        """
        if self.health_modal_handle_thread:  # 健康弹窗仍在监听的话，需要断掉，不然会报错
            self.stop_check = True
            self._wait(lambda: self.check_done, timeout=20, interval=2)
        self.app and self.app.session.close()

    def get_authorize_settings(self):
        """
        todo @locker
        :return:
        """
        pass

    def back_from_authorize_setting(self):
        """
        todo @locker
        :return:
        """
        self.app.session(class_name="Button", text="返回").click(timeout=10.0)

    def authorize_page_checkbox_enable(self, name, enable):
        """
        todo @locker
        :return:
        """
        pass

    @staticmethod
    def _alert_callback(session):
        """
        auto accept when system alert view popup
        :return:
        """
        if session.alert.exists:
            logger.info("出现弹框, 默认接受")
            session.alert.accept()

    def _perf_callback(self, data_type: str, value: dict):
        # logger.debug(f"@_perf_callback {data_type}, {value}")
        if data_type not in ("cpu", "fps", "memory"):
            return
        timestamp = int(value["timestamp"] / 1000)  # value["timestamp"] is ms
        if len(self.perf_data) > 0 and self.perf_data[-1]["timestamp"] == timestamp:
            item = self.perf_data[-1]
        else:
            item = {"timestamp": timestamp}
            self.perf_data.append(item)
        if data_type == "cpu":
            item.update(
                {"mem": self.last_mem, "cpu": value["value"], "fps": self.last_fps}
            )
            self.last_cpu = value["value"]
        elif data_type == "fps":
            item.update(
                {"mem": self.last_mem, "cpu": self.last_cpu, "fps": value["value"]}
            )
            self.last_fps = value["value"]
        elif data_type == "memory":
            item.update(
                {"mem": value["value"], "cpu": self.last_cpu, "fps": self.last_fps}
            )
            self.last_mem = value["value"]

    def start_get_perf(self, timeinterval=15):
        """
        开始获取性能数据
        :param timeinterval: 抽样时间间隔
        :return: boolen
        """
        if self.perf_flag:
            return True
        global tidevice
        if not self.useTIDevice:
            return False
        if not tidevice:
            import tidevice
        t = tidevice.Device(self.udid)
        self.perf = tidevice.Performance(t)
        self.last_fps = 0
        self.last_cpu = 0
        self.last_mem = 0
        self.perf.start(self.bundle_id, callback=self._perf_callback)
        self.perf_flag = True
        return self.perf_flag

    def stop_get_perf(self):
        """
        停止获取性能数据
        :return: string: json.dumps([{cpu, mem, fps, timestamp}])
        """
        self.perf_flag = False
        if not self.perf:
            return ""
        self.perf.stop()
        result = json.dumps(self.perf_data)
        self.perf_data = []
        self.last_fps = 0
        self.last_cpu = 0
        self.last_mem = 0
        return result


if __name__ == "__main__":
    native = WXIOSNative(
        {
            "device_info": {
                # "udid": "aee531018e668ff1aadee0889f5ebe21a2292...",
                # "model": "iPhone XR",
                # "version": "12.2.5",
                # "name": "yopofeng's iPhone12"
            }
        }
    )
    native.start_wechat()
    try:
        native.pick_media_file(names=["照片6,拍摄时间,2021年3月22日 上午11:49"])
    finally:
        native.release()



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       __init__.py
Create time:    2019/11/28 15:59
Description:

"""
import typing
from .wx_minium import WXMinium
from .qq_minium import QQMinium


# application
APP_WX = "wx"
APP_QQ = "qq"

APP = {"wx": WXMinium, "qq": QQMinium}

Minium = typing.Union[WXMinium, QQMinium]
def get_minium_driver(conf, *args, **kwargs) -> Minium:
    if conf is None:
        conf = {}
    application = conf.get("app", APP_WX)
    if application not in APP.keys():
        raise RuntimeError("the 'app' in your config file is not in predefine, not support yet")
    return APP[application](conf, *args, **kwargs)



#!/usr/bin/env python
# encoding:utf-8
import json
import logging
import random
import time

import at.utils.apkinfo
from attestbase import AtTestBase

logger = logging.getLogger()


class AtDeviceTest(AtTestBase):
    def setUp(self):
        super(AtDeviceTest, self).setUp()
        self.device = self.at.device

    def tearDown(self) -> None:
        pass  # self.test_dump()

    def test_enter(self):
        goods = {"goods_id": 66, "env": "beta", "api_id": "26345264",
                 "goods_order": {"env": "beta", "http_heads": {"WepayTest": "apitool", "Wepaytest-Uin": "3190375045"},
                                 "ip": "", "port": "",
                                 "req": {"_sign_key": "2480150211@APIKEY#test0123456789", "appid": "wxac5f4027faf15924",
                                         "attach": "attach", "body": "body", "detail": {"cost_price": "6",
                                                                                        "goods_detail": [
                                                                                            {"goods_id": "1",
                                                                                             "goods_name": "\u5546\u54c15678",
                                                                                             "price": "5",
                                                                                             "quantity": "1",
                                                                                             "wxpay_goods_id": "5678"},
                                                                                            {"goods_id": "1",
                                                                                             "goods_name": "\u5546\u54c15678",
                                                                                             "price": "1",
                                                                                             "quantity": "1",
                                                                                             "wxpay_goods_id": "5678"}],
                                                                                        "receipt_id": "wx123"},
                                         "device_info": "iphone", "fee_type": "CNY", "goods_tag": "",
                                         "mch_id": "2480150211", "nonce_str": "",
                                         "notify_url": "http://9.2.165.87:37845/fake/revcpaynotifyv2", "openid": "",
                                         "out_trade_no": "autotest_20210720162036_60882", "pay_sign_id": "",
                                         "product_id": "", "profit_sharing": "", "receipt": "", "risk_info": "",
                                         "risk_level": "", "scene_info": {
                                         "store_info": {"address": "ddf", "area_code": "addf", "id": "id",
                                                        "name": "name"}}, "sign": "", "sign_type": "",
                                         "spbill_create_ip": "", "sub_mch_id": "", "time_expire": "", "time_start": "",
                                         "total_fee": 5.0, "trade_type": "JSAPI", "version": "", "user_truename": " "},
                                 "timeout": 6, "uri": "/pay/unifiedorder"}}
        s = json.dumps(goods)
        self.at.e.edit_text().click()
        while s:
            sub_str = s[:128]
            s = s[128:]
            text = sub_str.replace('"', '\"')
            self.adb.run_shell("am broadcast -a ADB_INPUT_TEXT --es msg '%s'" % text)

    def test_gesture(self):
        width = self.device.width()
        height = self.device.height()
        points = [
            (int(width * 0.1), int(0 + height * 0.5)),
            (int(width * 0.5), int(height * 0.1)),
            (int(width * 0.9), int(height * 0.5)),
            (int(width * 0.5), int(height * 0.9)),
            (int(width * 0.1), int(height * 0.5)),
        ]
        self.device.gesture(points)

    def test_screen(self):
        width = self.device.width()
        height = self.device.height()
        self.device.screen_shot("test.png")
        self.device.screen_point("half.png", 0, 0, width/2, height/2)

    def test_scroll(self):
        self.device.scroll_one_forth_page("up", 1)
        self.device.scroll_one_forth_page("up", 1)
        self.device.scroll_one_forth_page("down", 1)
        self.device.scroll_one_forth_page("down", 1)

    def test_many_dump(self):
        logging.basicConfig(level=logging.INFO)
        total = 0
        for i in range(100):
            s = time.time()
            content = self.at.java_driver.dump_all_views()
            costs = time.time() - s

            if costs > 5:
                logger.info("异常:%s, %s", i, costs)
            total += costs
        logger.info("平均耗时:%s", total / 100)

    def test_get_close_btn(self):
        self.at.java_driver.request_configure('setUseCustom', [True])
        self.at.java_driver.request_configure('setBlockKindaMain', [True])
        text = self.at.e.rid("com.tencent.mm:id/closeModalBtn").get_desc()
        print(text)

    def test_get_current_activity(self):
        act = self.adb.get_current_activity()
        print(act)

    def test_dump_diff_visisble(self):
        # self.at.java_driver.request_configure('setIgnoreNotVisibleNode', [True])
        # logger.info(f"{'='*40}ignoreNotVisibleNode=true{'='*40}")
        # self.test_dump()
        self.at.java_driver.request_configure('setIgnoreNotVisibleNode', [False])
        logger.info(f"{'=' * 40}ignoreNotVisibleNode=false{'=' * 40}")
        self.test_dump()

    def test_dump(self):
        index = 0
        # self.at.java_driver.remove_kinda_main(True)
        for ui_views in self.at.java_driver.dump_all_views():
            logging.info("%s index=%s %s", "=" * 60, index, "=" * 60)
            index += 1
            for ui_view in ui_views:
                logging.info(ui_view)
        # self.at.java_driver.reconnect()
        # for ui_views in self.at.java_driver.dump_all_views():
        #     logging.info("=" * 120)
        #     for ui_view in ui_views:
        #         logging.info(ui_view)

    def test_long_time_dump(self):
        # logger.setLevel(logging.INFO)
        t = time.time()
        index = 1
        width = self.at.device.width()
        height = self.at.device.height() - 100
        while time.time() - t < 60 * 60:
            self.at.device.click_on_point(random.randint(0, width), random.randint(0, width))
            if random.random() < 0.1:
                self.adb.press_back()
            elif random.random() < 0.1:
                self.adb.press_home()
            ui_views_list = self.at.java_driver.dump_all_views()
            count = 0
            for ui_views in ui_views_list:
                count += len(ui_views)
            logger.info(f"{index} window count:{len(ui_views)}, view count:{count}")
            index += 1

    def test_finish(self):

        self.at.e.text('1').click()

    def test_height(self):
        height = self.at.device.height()
        status_height = self.at.device.status_bar_height()
        logging.info("%s, %s", height, status_height)

    def test_parser(self):
        ai = at.utils.apkinfo.ApkInfo("/Users/xiazeng/Downloads/wxpayface-lpos-release-2.22.100.46-signed.apk")
        print(ai.to_dict())

    def test_window_handle(self):
        window_handle_general = r"(^(完成|关闭|关闭应用|好|允许|始终允许|好的|确定|确认|安装|下次再说|知道了|同意)$|(.*((?<!不)(忽略|允(\s){0,2}许|同(\s){0,2}意)|继续|清理|稍后|稍后处理|暂不|暂不设置|强制|下一步)).*)|^((?i)allow|Sure|SURE|accept|install|done|ok)$"
        window_handle_general = r"(^(完成|关闭|关闭应用|好|允许|始终允许|好的|确定|确认|安装|下次再说|知道了|同意)$|(.*((?<!不)(忽略|允(\s){0,2}许|同(\s){0,2}意)|继续|始终允许|清理|稍后|稍后处理|暂不|暂不设置|强制|下一步)).*)"

        self.at.event_monitor.add_event_click(self.at.e.text_matches(window_handle_general).clickable(True),
                                              self.at.e.text_matches(window_handle_general).clickable(True))
        # for sp in window_handle_special:
        #     self.at.event_monitor.add_event_click(self.at.e.text_matches(sp[0]), self.at.e.text_matches(sp[1]))
        time.sleep(60)

    def test_event_monitor(self):
        self.at.event_monitor.add_selector_filter("preftest_permission1", self.at.e.text_contains("允许微信拍摄"),
                                                  self.at.e.text("仅在使用该应用时允许"))
        self.at.event_monitor.add_selector_filter("preftest_permission2", self.at.e.text_contains("允许微信录音"),
                                                  self.at.e.text("仅在使用该应用时允许"))
        self.at.event_monitor.add_selector_filter("preftest_permission3", self.at.e.text_contains("允许微信获取"),
                                                  self.at.e.text("仅在使用该应用时允许"))
        time.sleep(40)

    def test_is_exists(self):
        click_count = 0
        for i in range(100):
            if self.at.e.text("发现").click_if_exists():
                click_count += 1
        print("success click:", click_count)

    def test_click(self):
        while True:
            self.at.device.click_on_point(1000, 2300)
            time.sleep(0.02)

    def test_get_current_package(self):
        for i in range(1000):
            logger.info(self.device.current_package())
            time.sleep(1)

    def test_clickNOn(self):
        self.at.e.text("zx").click()

    def test_enter(self):
        self.at.device.enter("adaldagdad")

    def test_logcat(self):
        self.at.logcat.start_record("test1", "MicroMsg.DynamicConfig")
        self.at.logcat.start_record("test2", "MicroMsg.KeyBordUtil")
        self.at.logcat.start_record("test3", "MicroMsg.RoomServiceFactory")
        for i in range(1):
            self.at.adb.stop_app("com.tencent.mm")
            self.at.adb.start_app("com.tencent.mm", "ui.LauncherUI")
            time.sleep(10)
        for name in self.at.logcat.filter_map:
            lines = self.at.logcat.get_lines(name)
            logger.info("%s, lines num:%s", name, len(lines))



#!/usr/bin/env python3
# Created by xiazeng on 2019-05-22
import os.path
import sys
import typing

work_root = os.path.abspath(os.path.dirname(__file__))
lib_path = os.path.join(work_root, "lib")
sys.path.insert(0, lib_path)

from minium.framework.miniconfig import MiniConfig, logger
from .exception import *

# # weChat
from minium.native.wx_native.basenative import BaseNative

# from minium.native.wx_native.androidnative import WXAndroidNative
# from minium.native.wx_native.iosnative import WXIOSNative
# from minium.native.wx_native.idenative import IdeNative

# # QQ
# from minium.native.qq_native.qandroidnative import QAndroidNative
# from minium.native.qq_native.qiosnative import QiOSNative

# platform
from minium.utils.platforms import *
# OS_ANDROID = "android"
# OS_IOS = "ios"
# OS_IDE = "ide"
# OS_MAC = "mac"
# OS_WIN = "win"

# application
APP_WX = "wx"
APP_QQ = "qq"


class APPS(dict):
    def __getitem__(self, __k):
        try:
            v = super().__getitem__(__k)
        except KeyError:
            v = None
        if v is not None:
            return v
        if __k == "wx_android":
            from minium.native.wx_native.androidnative import WXAndroidNative

            super().__setitem__(__k, WXAndroidNative)
            return WXAndroidNative
        elif __k == "wx_ios":
            from minium.native.wx_native.iosnative import WXIOSNative

            super().__setitem__(__k, WXIOSNative)
            return WXIOSNative
        elif __k == "ide":
            from minium.native.wx_native.idenative import IdeNative

            super().__setitem__(__k, IdeNative)
            return IdeNative
        elif __k == "qq_android":
            from minium.native.qq_native.qandroidnative import QAndroidNative

            super().__setitem__(__k, QAndroidNative)
            return QAndroidNative
        elif __k == "qq_ios":
            from minium.native.qq_native.qiosnative import QiOSNative

            super().__setitem__(__k, QiOSNative)
            return QiOSNative
        raise KeyError("APPS not support %s" % __k)


APP = APPS()


def get_native_driver(os_name, conf, *args):
    if os_name.lower() not in [OS_ANDROID, OS_IDE, OS_IOS]:
        raise RuntimeError("the 'os_name' in your config file is not in predefine")
    if os_name.lower() != OS_IDE and conf.get("app", None) not in [APP_WX, APP_QQ]:
        raise RuntimeError(
            f"the 'app': '{os_name}' in your config file is not in predefine, not support yet"
        )
    if os_name.lower() == OS_IDE:
        return APP[os_name.lower()]({}, *args)
    elif conf.device_desire is None:
        logger.warning(
            "your platform is [{}], but dosn't configure the [device_desire] field, native"
            " interface will not in use!".format(os_name)
        )
        return APP[OS_IDE]({})
    else:
        json_conf = {"outputs": conf.outputs, "debug": conf.debug or False}
        json_conf.update(conf.device_desire or {})
        return APP[conf.app.lower() + "_" + os_name.lower()](json_conf)


def Native(json_conf, platform=None, app="wx"):
    cfg = MiniConfig({"platform": platform, "app": app, "device_desire": json_conf})
    return get_native_driver(platform, cfg)



# -*- coding: utf-8 -*-
# Created by xiazeng on 2020/3/10
import time
import logging
from attestbase import AtTestBase
from at.core.stf.mincap import MiniCap

logger = logging.getLogger()


class MiniCapTest(AtTestBase):
    def setUp(self):
        self.front = MiniCap(display_id=1)
        self.back = MiniCap(display_id=0)

    def test_screenshot(self):
        open("front.jpg", "wb").write(self.front.get_frame())
        open("back.jpg", "wb").write(self.back.get_frame())


#!/usr/bin/env python3

"""
Author:         xiazeng
Create time:    2021/5/20
Description:    

"""
import logging
import time
import unittest

logging.basicConfig(level=logging.DEBUG,
                    format="%(levelname)-5.5s %(asctime)s %(filename)s %(funcName)s %(lineno)-3d %(message)s")

logger = logging.getLogger()

import uiautomator2


class U2Test(unittest.TestCase):
    def setUp(self) -> None:
        self.d = uiautomator2.connect()

    def test_dump(self):
        total = 0
        for i in range(100):
            s = time.time()
            content = self.d.dump_hierarchy()
            costs = time.time() - s
            if costs > 5:
                logger.info("异常:%s, %s", i, costs)
            total += costs
        logger.info("平均耗时:%s", total/100)
